@@ -151,15 +151,16 @@ class Attention2d(torch.nn.Module):
         self.act = act_layer()
         self.proj = ConvNorm(self.dh, dim, 1, **dd)
 
-        pos = torch.stack(ndgrid(
-            torch.arange(self.resolution[0], device=device, dtype=torch.long),
-            torch.arange(self.resolution[1], device=device, dtype=torch.long),
-        )).flatten(1)
-        rel_pos = (pos[..., :, None] - pos[..., None, :]).abs()
-        rel_pos = (rel_pos[0] * self.resolution[1]) + rel_pos[1]
-        self.attention_biases = torch.nn.Parameter(torch.zeros(num_heads, self.N, **dd))
-        self.register_buffer('attention_bias_idxs', rel_pos, persistent=False)
-        self.attention_bias_cache = {}  # per-device attention_biases cache (data-parallel compat)
+        self.attention_biases = torch.nn.Parameter(torch.empty(num_heads, self.N, **dd))
+        self.register_buffer(
+            'attention_bias_idxs',
+            torch.empty((self.N, self.N), device=device, dtype=torch.long),
+            persistent=False,
+        )
+        self.attention_bias_cache = {}
+
+        if not self.attention_biases.is_meta:
+            self.reset_parameters()
 
     @torch.no_grad()
     def train(self, mode=True):
@@ -167,6 +168,32 @@ class Attention2d(torch.nn.Module):
         if mode and self.attention_bias_cache:
             self.attention_bias_cache = {}  # clear ab cache
 
+    def reset_parameters(self) -> None:
+        """Initialize parameters and buffers."""
+        nn.init.zeros_(self.attention_biases)
+        self._init_buffers()
+
+    def _compute_attention_bias_idxs(self, device=None):
+        """Compute relative position indices for attention bias."""
+        pos = torch.stack(ndgrid(
+            torch.arange(self.resolution[0], device=device, dtype=torch.long),
+            torch.arange(self.resolution[1], device=device, dtype=torch.long),
+        )).flatten(1)
+        rel_pos = (pos[..., :, None] - pos[..., None, :]).abs()
+        rel_pos = (rel_pos[0] * self.resolution[1]) + rel_pos[1]
+        return rel_pos
+
+    def _init_buffers(self) -> None:
+        """Compute and fill non-persistent buffer values."""
+        self.attention_bias_idxs.copy_(
+            self._compute_attention_bias_idxs(device=self.attention_bias_idxs.device)
+        )
+        self.attention_bias_cache = {}
+
+    def init_non_persistent_buffers(self) -> None:
+        """Initialize non-persistent buffers."""
+        self._init_buffers()
+
     def get_attention_biases(self, device: torch.device) -> torch.Tensor:
         if torch.jit.is_tracing() or self.training:
             return self.attention_biases[:, self.attention_bias_idxs]
@@ -265,7 +292,30 @@ class Attention2dDownsample(torch.nn.Module):
         self.act = act_layer()
         self.proj = ConvNorm(self.dh, self.out_dim, 1, **dd)
 
-        self.attention_biases = nn.Parameter(torch.zeros(num_heads, self.N, **dd))
+        self.attention_biases = nn.Parameter(torch.empty(num_heads, self.N, **dd))
+        self.register_buffer(
+            'attention_bias_idxs',
+            torch.empty((self.N2, self.N), device=device, dtype=torch.long),
+            persistent=False,
+        )
+        self.attention_bias_cache = {}
+
+        if not self.attention_biases.is_meta:
+            self.reset_parameters()
+
+    @torch.no_grad()
+    def train(self, mode=True):
+        super().train(mode)
+        if mode and self.attention_bias_cache:
+            self.attention_bias_cache = {}  # clear ab cache
+
+    def reset_parameters(self) -> None:
+        """Initialize parameters and buffers."""
+        nn.init.zeros_(self.attention_biases)
+        self._init_buffers()
+
+    def _compute_attention_bias_idxs(self, device=None):
+        """Compute relative position indices for attention bias."""
         k_pos = torch.stack(ndgrid(
             torch.arange(self.resolution[0], device=device, dtype=torch.long),
             torch.arange(self.resolution[1], device=device, dtype=torch.long),
@@ -276,14 +326,18 @@ class Attention2dDownsample(torch.nn.Module):
         )).flatten(1)
         rel_pos = (q_pos[..., :, None] - k_pos[..., None, :]).abs()
         rel_pos = (rel_pos[0] * self.resolution[1]) + rel_pos[1]
-        self.register_buffer('attention_bias_idxs', rel_pos, persistent=False)
-        self.attention_bias_cache = {}  # per-device attention_biases cache (data-parallel compat)
+        return rel_pos
 
-    @torch.no_grad()
-    def train(self, mode=True):
-        super().train(mode)
-        if mode and self.attention_bias_cache:
-            self.attention_bias_cache = {}  # clear ab cache
+    def _init_buffers(self) -> None:
+        """Compute and fill non-persistent buffer values."""
+        self.attention_bias_idxs.copy_(
+            self._compute_attention_bias_idxs(device=self.attention_bias_idxs.device)
+        )
+        self.attention_bias_cache = {}
+
+    def init_non_persistent_buffers(self) -> None:
+        """Initialize non-persistent buffers."""
+        self._init_buffers()
 
     def get_attention_biases(self, device: torch.device) -> torch.Tensor:
         if torch.jit.is_tracing() or self.training:
@@ -664,15 +718,21 @@ class EfficientFormerV2(nn.Module):
         else:
             self.head_dist = None
 
-        self.apply(self.init_weights)
+        if not self.norm.weight.is_meta:
+            self.init_weights(needs_reset=False)
+
         self.distilled_training = False
 
-    # init for classification
-    def init_weights(self, m):
+    def _init_weights(self, m, needs_reset: bool = True):
         if isinstance(m, nn.Linear):
             trunc_normal_(m.weight, std=.02)
             if m.bias is not None:
                 nn.init.constant_(m.bias, 0)
+        elif needs_reset and hasattr(m, 'reset_parameters'):
+            m.reset_parameters()
+
+    def init_weights(self, needs_reset: bool = True):
+        self.apply(partial(self._init_weights, needs_reset=needs_reset))
 
     @torch.jit.ignore
     def no_weight_decay(self):
